package dev.fyloz.colorrecipesexplorer.config.security.filters

import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.properties.CreSecurityProperties
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.dtos.account.UserLoginRequestDto
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import org.springframework.security.authentication.AuthenticationManager
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.Authentication
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

val blacklistedJwtTokens = mutableListOf<String>()

class UsernamePasswordAuthenticationFilter(
    jwtLogic: JwtLogic,
    securityProperties: CreSecurityProperties,
    private val authManager: AuthenticationManager,
    private val updateUserLoginTime: (Long) -> Unit
) : JwtAuthenticationFilter(Constants.ControllerPaths.LOGIN, securityProperties, jwtLogic) {
    override fun attemptAuthentication(request: HttpServletRequest, response: HttpServletResponse): Authentication {
        val loginRequest = getLoginRequest(request)
        val authenticationToken = UsernamePasswordAuthenticationToken(loginRequest.id, loginRequest.password)

        logger.debug("Login attempt for user ${loginRequest.id}")
        return authManager.authenticate(authenticationToken)
    }

    override fun afterSuccessfulAuthentication(userDetails: UserDetails) {
        updateUserLoginTime(userDetails.id.toLong())
        logger.info("User ${userDetails.id} (${userDetails.username}) has logged in successfully")
    }

    private fun getLoginRequest(request: HttpServletRequest) =
        jacksonObjectMapper().readValue(request.inputStream, UserLoginRequestDto::class.java)
}