package dev.fyloz.colorrecipesexplorer.logic.account

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.LogicComponent
import dev.fyloz.colorrecipesexplorer.dtos.account.GroupDto
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDto
import dev.fyloz.colorrecipesexplorer.logic.BaseLogic
import dev.fyloz.colorrecipesexplorer.logic.Logic
import dev.fyloz.colorrecipesexplorer.service.account.GroupService
import org.springframework.transaction.annotation.Transactional

interface GroupLogic : Logic<GroupDto, GroupService> {
    /** Gets all the users of the group with the given [id]. */
    fun getUsersForGroup(id: Long): Collection<UserDto>
}

@LogicComponent
class DefaultGroupLogic(service: GroupService, private val userLogic: UserLogic) :
    BaseLogic<GroupDto, GroupService>(service, Constants.ModelNames.GROUP),
    GroupLogic {
    override fun getUsersForGroup(id: Long) = userLogic.getAllByGroup(getById(id))

    @Transactional
    override fun save(dto: GroupDto): GroupDto {
        throwIfNameAlreadyExists(dto.name)

        return super.save(dto)
    }

    override fun update(dto: GroupDto): GroupDto {
        throwIfNameAlreadyExists(dto.name, dto.id)

        return super.update(dto)
    }

    private fun throwIfNameAlreadyExists(name: String, id: Long? = null) {
        if (service.existsByName(name, id)) {
            throw alreadyExistsException(value = name)
        }
    }
}
